#!/bin/sh
# Script to install the OCALA proxy (must be run with root permissions.)
# Author: Dilip Antony Joseph
# NOTE: This script has been tested only with Fedora Linux. 
# If it works for you for some other version of Linux, please send a mail to support@ocala.cs.berkeley.edu.
#
# Reference: http://www.linux.com/howtos/HighQuality-Apps-HOWTO/boot.shtml
#
OCALA_INSTALL_DIR=/usr/local/OCALA

# The configuration files used by the installed version of the OCALA proxy are in 
# following directory.
OCALA_CONFIG_DIR=/usr/local/OCALA
# Note: DILIP: OCALA_CONFIG_DIR must be the same as OCALA_CONFIG_DIR, for now.  
# This is because OCALA_proxy is not very intelligent while looking for the directories
# containing the configuration files.

# Set the following variable to blank if you do not want to be prompted for file overwrite.
PROMPT_OVERWRITE=-i

# Check if the tun device has already been installed.
./checkTun.pl
if [ $? -eq 2 ]
then
    echo
    echo "Tun device not installed.  You cannot continue with the installation."
    echo "Please run OCALA/configure_tun.pl followed by installOCALA.sh, both with superuser privileges."
    echo
    exit 2
fi

# We need superuser permissions. Check for that.
if [ $UID -ne 0 ] 
then
    echo
    echo "You need superuser permissions to install OCALA as a service. Aborting."
    echo "Please run ./installOCALA.sh with superuser permissions."
    echo
    exit -1
fi

# Create the OCALA installation directory if it doesn't already exist.
if [ ! -d $OCALA_INSTALL_DIR ]
then
    mkdir $OCALA_INSTALL_DIR
fi
# Check if the OCALA installation directory was successfully created or existed previously.
if ! [ -d $OCALA_INSTALL_DIR ]  
then
    echo "Unable to create $OCALA_INSTALL_DIR.  Aborting installation."
    exit 3
fi

# Create the OCALA configuration directory if it doesn't already exist.
if [ ! -d $OCALA_CONFIG_DIR ]
then
    mkdir $OCALA_CONFIG_DIR
fi
if [ ! -d $OCALA_CONFIG_DIR/config_files ]
then
    mkdir $OCALA_CONFIG_DIR/config_files
fi
if [ ! -d $OCALA_CONFIG_DIR/config_files/certificates ]
then
    mkdir $OCALA_CONFIG_DIR/config_files/certificates
fi


# Check if the OCALA configuration directory was successfully created or existed previously.
if ! [ -d $OCALA_CONFIG_DIR ]  
then
    echo "Unable to create $OCALA_CONFIG_DIR.  Aborting installation."
    exit 3
fi

echo "Installing OCALA to $OCALA_INSTALL_DIR."

# Copy the main OCALA proxy executable.
if [ ! -f OCALA/generic_proxy/OCALA_proxy ]
then
    echo "Unable to find OCALA_proxy executable in OCALA/generic_proxy."
    echo "Did you forget to do a ../compile from inside the subdirectory OCALA?"
    echo "Aborting installation."
    exit 4
fi
cp $PROMPT_OVERWRITE OCALA/generic_proxy/OCALA_proxy $OCALA_INSTALL_DIR

# The following file is required for requesting a certificate for a name.
cp $PROMPT_OVERWRITE OCALA/generic_proxy/security/nameallocator/nameAllocatorClient $OCALA_INSTALL_DIR

# Copy the various OCD modules
if [ ! -f i3/i3OCD/libI3OCD.so ]
then
    echo "Unable to find i3 OCD module."
    echo "Did you forget to do a ../compile from inside the subdirectory i3?"
    echo "Aborting installation."
    exit 4
fi
cp $PROMPT_OVERWRITE i3/i3OCD/libI3OCD.so $OCALA_INSTALL_DIR

if [ ! -f sampleOCD/libSampleOCD.so ]
then
    echo "Unable to find SampleOCD OCD module."
    echo "Did you forget to do a ../compile from inside the subdirectory sampleOCD?"
    echo "Aborting installation."
    exit 4
fi
cp $PROMPT_OVERWRITE sampleOCD/libSampleOCD.so $OCALA_INSTALL_DIR

if [ ! -f RSPTunnelOCD/libRSPTunnelOCD.so ]
then
    echo "Unable to find RSPTunnelOCD module."
    echo "Did you forget to do a ../compile from inside the subdirectory RSPTunnelOCD?"
    echo "Aborting installation."
    exit 4
fi
cp $PROMPT_OVERWRITE RSPTunnelOCD/libRSPTunnelOCD.so $OCALA_INSTALL_DIR

# Copy the ProxyConfig to the INSTALL_DIR too
if [ ! -f OCALA/config_gui/ProxyConfig.jar ] 
then
    echo "Unable to find ProxyConfig.jar.  Perhaps you do not have Java installed?"
    echo "Graphical configuration utility will be unavailable."
    echo
else
    cp OCALA/config_gui/ProxyConfig.jar $OCALA_INSTALL_DIR
fi


# Copy the current configuration files to the service directory
CONFIG_FILE_SRC=OCALA/generic_proxy/config_files
if  [       ! -d $CONFIG_FILE_SRC  \
        -o  ! -f $CONFIG_FILE_SRC/proxy-config.xml \
        -o  ! -f $CONFIG_FILE_SRC/myNames.xml \
        -o  ! -f $CONFIG_FILE_SRC/outbound.xml \
        -o  ! -f $CONFIG_FILE_SRC/tun-config.xml \
        -o  ! -f $CONFIG_FILE_SRC/i3-cfg.xml \
        -o  ! -f $CONFIG_FILE_SRC/i3_addr_book.txt \
        -o  ! -f $CONFIG_FILE_SRC/sampleOCD.xml \
        -o  ! -f $CONFIG_FILE_SRC/RSPTunnelOCD.xml \
        -o  ! -f $CONFIG_FILE_SRC/certificates/CA_cert.pem \
    ]
then
    echo "Unable to find some of the required configuration files in $CONFIG_FILE_SRC. Aborting installation."
    exit 5
fi

echo "Copying configuration files to $OCALA_CONFIG_DIR."
cp $PROMPT_OVERWRITE $CONFIG_FILE_SRC/proxy-config.xml \
                        $CONFIG_FILE_SRC/myNames.xml \
                        $CONFIG_FILE_SRC/outbound.xml \
                        $CONFIG_FILE_SRC/tun-config.xml \
                        $CONFIG_FILE_SRC/i3-cfg.xml \
                        $CONFIG_FILE_SRC/i3_addr_book.txt \
                        $CONFIG_FILE_SRC/sampleOCD.xml \
                        $CONFIG_FILE_SRC/RSPTunnelOCD.xml \
                $OCALA_CONFIG_DIR/config_files

cp $PROMPT_OVERWRITE $CONFIG_FILE_SRC/certificates/CA_cert.pem \
                $OCALA_CONFIG_DIR/config_files/certificates




                
# Copy and install the OCALA start/stop script to the /etc/init.d directory.
# This is done in order to enable OCALA to be started up as a service at boot time.
if [ ! -f OCALA_init.d ]
then
    echo "Required OCALA_init.d script missing. Aborting installation."
    exit
fi

cp $PROMPT_OVERWRITE OCALA_init.d /etc/init.d/OCALA
# Run chkconfig to enable automatic starting of OCALA at boot time.

# Check if chkconfig command can be located.
CHKCONFIG_CMD=chkconfig
which chkconfig 2>/dev/null 1>/dev/null
if [ $? -gt 0 ]
then
    # Check /sbin/chkconfig
    if [ ! -e "/sbin/chkconfig" ]
    then 
        echo "Required command chkconfig not found in your PATH. Please make sure you use \"su -\" instead of \"su\". Aborting installation."
        exit
	else 
		CHKCONFIG_CMD=/sbin/chkconfig
    fi
fi

#Run chkconfig
$CHKCONFIG_CMD --add OCALA 2>&1> /dev/null
if [ ! $? ]
then
    echo "chkconfig failed. Unable to make OCALA automatically start at boot time."
    exit 6
fi

# copy the launch proxy config gui script to the install directory
cp $PROMPT_OVERWRITE OCALA/config_gui/configureOCALA.sh $OCALA_INSTALL_DIR

which java 2>/dev/null 1>/dev/null
if [ $? -eq 0 ] && [ -f $OCALA_INSTALL_DIR/ProxyConfig.jar ]
then
   	echo "Launching Java ProxyConfig GUI ..."
    echo
    cd $OCALA_INSTALL_DIR
    java -jar ProxyConfig.jar
    cd -
else
    echo "java or ProxyConfig.jar not found.  Launching limited command line configuration..."
    echo

	ans="invalid"
	while [ $ans != "y"  -a  $ans != "Y"  -a   $ans != "N"   -a  $ans != "n" ]
	do
    	echo "Do you want to set an i3 name for this machine? (yYnN)"
    	read ans
		if [ ! $ans ]
		then
			ans="invalid"
		fi
	done

	if [ $ans = "y"  -o  $ans = "Y" ]
	then
    	./fillInMyName.sh $OCALA_CONFIG_DIR/config_files/myNames.xml
	fi
fi
 





echo
echo 
echo
echo "OCALA successfully installed into $OCALA_INSTALL_DIR."
echo
echo
echo "This installation uses the configuration files in $OCALA_CONFIG_DIR."
echo "You can graphically configure OCALA by running the following command inside the $OCALA_CONFIG_DIR:"
echo "configureOCALA.sh"
echo
echo "To start the OCALA service now, use the command: service OCALA start"
echo

exit 0
